;***********************************
;* Universal Remote Control Reader *
;*                                 *
;* Ken Kaarvik    Mar13/99         *
;*                                 *
;***********************************


;Description of what you see on the GBC screen
;
;aa       m1 s1 17 17
;         m2 s2 18 18
;bb       m3 s2 19 19
;         m4 s4 20 20
;cc       m5 s5 21 21
;         m6 s6 22 22
;dd       m7 s7 23 23
;         m8 s8 24 24
;ee       m9 s9 25 25
;         10 10 26 26
;         11 11 27 27
;         12 12 28 28
;  S1 S0  13 13 29 29
;         14 14 30 30
;         15 15 31 31
;   W0    16 16 32 32
;web_bits
; 0123456 P
;
;Description of above
;aa=counter - inc's every write to screen
;bb=joypad
;cc=carrier off (always 1)
;dd=carrier on 07=40kHz? 09=32kHz?
;ee=number of marks (was duty cycle)
;m1=first mark, num of cycles per mark (header)
;s1=first space measured in software loops
;s2-32,m2-32 next 31
;S0=decoded byte for sony (m9 m8 m7 m6 m5 m4 m3 m2)
;S1                       (17 16 15 14 13 12 11 10)
;
;W0
;  Sony web bits in hex see below
;
;web_bits:
;  all the buttons on Sony's web tv keyboard start the same
;  m1=56,s1=1D,m2=1E,s2=1D,m3=3C,s3=1D,m4=1E,s4=1D
;  The waveform shows the following marks and spaces in relative
;   lengths ( one tile = 1E)
;
;Operation
; press A button (screen will go red)
;    wait for infrared data
;    all data on screen will update
; press B button (screen will go light red)
;    send out stored data
;
; press Sel (screen will go blue)
;    save recorded data into 1 of 4 cart ram locations
;    press d pad to write to cart ram
;
; press d pad to send saved data
;
; if the header is correct for sony then update S1 S0
;   else S1=99

web_mark_symbol		equ	17
web_space_symbol	equ	18
web_line		equ	$9800+$20*16

cr_length	equ	$100
d_up_r		equ	$A000
d_down_r	equ	d_up_r+cr_length
d_left_r	equ	d_down_r+cr_length
d_right_r	equ	d_left_r+cr_length

	SECTION "Header",HOME[$0100]

	nop
	jp	Startup

	DB	$CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
	DB	$00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
	DB	$BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E

		;0123456789ABCDE
	DB	"UniversalRemote"
	DB	$80	;$80=Color GB
	DB	0,0,0	;SuperGameboy
	DB	3	;CARTTYPE
			;--------
			;0 - ROM ONLY
			;1 - ROM+MBC1
			;2 - ROM+MBC1+RAM
			;3 - ROM+MBC1+RAM+BATTERY
			;5 - ROM+MBC2
			;6 - ROM+MBC2+BATTERY

	DB	0	;ROMSIZE
			;-------
			;0 - 256 kBit ( 32 kByte,  2 banks)
			;1 - 512 kBit ( 64 kByte,  4 banks)
			;2 -   1 MBit (128 kByte,  8 banks)
			;3 -   2 MBit (256 kByte, 16 banks)
			;3 -   4 MBit (512 kByte, 32 banks)

	DB	2	;RAMSIZE
			;-------
			;0 - NONE
			;1 -  16 kBit ( 2 kByte, 1 bank )
			;2 -  64 kBit ( 8 kByte, 1 bank )
			;3 - 256 kBit (32 kByte, 4 banks)

	DW	$0000	;Manufacturer

	DB	0	;Version
	DB	0	;Complement check
	DW	0	;Checksum

	SECTION	"Main",home[$0150]

Startup
	ld	a,1		;double speed
	ldh	[$4D],a
	stop

	di
	ld	hl,$FFFE
	ld	sp,hl

	call	initialize
	call	clear_ram
	;call	test_load_cart_ram

	call	intial_display

	call	blank_web_bits

	ld	a,%11000000
	ldh	[$56],a

Main	

	call	counter_inc

	call	keypad

	call	get_infrared

	call	send_infrared

	call	send_d_pad

	call	save_to_cart_ram

	;call	test_save_cart_ram

	call	waitVB

	call	write_to_screen

	jp	Main

counter_inc
	ld	a,[counter]
	inc	a
	ld	[counter],a
	ret

send_d_pad
	ld	a,[joypad]
	cp	$0F
	ret	c

	;call	set_to_blue
	;ret

	and	$F0
	cp	$40
	jp	z,d_up_send
	cp	$80
	jp	z,d_down_send
	cp	$20
	jp	z,d_left_send
	cp	$10
	jp	z,d_right_send

d_up_send
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,d_up_r	;start address
	ld	de,m_s		;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram

	call	send_infrared_d_pad
	call	set_to_black
	ret

d_down_send
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,d_down_r	;start address
	ld	de,m_s		;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram

	call	send_infrared_d_pad
	call	set_to_black
	ret

d_left_send
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,d_left_r	;start address
	ld	de,m_s		;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram

	call	send_infrared_d_pad
	call	set_to_black
	ret

d_right_send
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,d_right_r	;start address
	ld	de,m_s		;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram

	call	send_infrared_d_pad
	call	set_to_black
	ret

test_load_cart_ram
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram

	ld	hl,d_up		;start address
	ld	de,counter	;end address
	ld	bc,$100		;length
	call	mem_Copy

	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram

test_save_cart_ram
	ld	a,[joypad]	;save to cart if Sel button
	cp	$04
	ret	nz

	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram

	ld	hl,counter	;start address
	ld	de,d_up		;end address
	ld	bc,$100		;length
	call	mem_Copy

	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram

	ret


save_to_cart_ram
	ld	a,[joypad]	;save to cart if Sel button
	cp	$04
	ret	nz
	call	set_to_blue
wait_for_d_pad
	call	keypad
	ld	a,[joypad]
	cp	$0F
	jp	c,wait_for_d_pad
	and	$F0
	cp	$40
	jp	z,d_up
	cp	$80
	jp	z,d_down
	cp	$20
	jp	z,d_left
	cp	$10
	jp	z,d_right

d_up
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,m_s		;start address
	ld	de,d_up_r	;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram
	call	set_to_black
	ret

d_down
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,m_s		;start address
	ld	de,d_down_r	;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram
	call	set_to_black
	ret

d_left
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,m_s		;start address
	ld	de,d_left_r	;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram
	call	set_to_black
	ret

d_right
	ld	a,$0A
	ld	[$0000],a	;enable access to cart ram
	ld	hl,m_s		;start address
	ld	de,d_right_r	;end address
	ld	bc,$100		;length
	call	mem_Copy
	ld	a,$00
	ld	[$0000],a	;prohibit access to cart ram
	call	set_to_black
	ret

send_infrared
	ld	a,[joypad]	;send IR if B button
	cp	$02
	ret	nz
send_infrared_d_pad
	ld	a,[m_s]		;check if 0
	cp	0
	ret	z
	cp	$50		;error check
	ret	nc

	ld	e,a

	call	set_to_yellow
	ld	c,$56
	;ld	a,%00000000	;not needed
	;ld	[c],a
	;ld	a,[m_s]
	;ld	e,a		;num of total mark
	ld	hl,mark1

send_loop
	ld	a,[first_pulse]
	ld	d,$33		;testing
	ld	a,1
	ld	[c],a


send_first_pulse
	;nop
	;nop
	;nop
	;nop
	dec	d
	jp	nz,send_first_pulse

send_new_mark
	ld	a,[hl+]		;mark in carrier cycles
	ld	d,a

send_mark

	xor	a
	ldh	[$56],a		;carrier_off
	ldh	[$56],a
	ldh	[$56],a
	ldh	[$56],a
	ld	a,1
	ldh	[$56],a		;carrier_on

	ld	a,[carrier_on]	;length of carrier_on ***
	dec	a
	ld	b,a

send_carrier_on
	nop			;need nop for mits tv
				;no nop for ge tv
	dec	b
	jp	nz,send_carrier_on

	dec	d
	jp	nz,send_mark

	dec	e
	jp	z,end_send_ir

send_space
	ld	a,[hl+]
	ld	d,a
send_space_loop
	REPT	10
	nop			;fine tune here
	nop			;for space count
	nop
	nop
	nop
	ENDR

	dec	d
	jp	nz,send_space_loop
	;jp	send_new_mark
	jp	send_loop

end_send_ir
	xor	a
	ld	[c],a
	call	set_to_black
	ret

get_infrared
	ld	a,[joypad]	;get IR if A button
	rr	a
	ret	nc

	call	set_to_red
	call	clear_ram
	ld	c,$56
	ld	a,%11000000
	ld	[c],a
	ld	b,$02
	ld	d,0
	ld	e,0
	ld	hl,first_pulse

wait_for_ir
	ld	a,[c]
	and	b
	jp	nz,wait_for_ir


first_pulse_on
	;inc	d
	ld	a,[c]
	and	b
	jp	z,first_pulse_on	;wait for no ir

first_pulse_off				;always 1 count
	inc	d
	ld	a,[c]
	and	b
	jp	nz,first_pulse_off	;wait for ir

carrier_on_count
	inc	e
	ld	a,[c]
	and	b
	jp	z,carrier_on_count	;wait for no ir



	ld	a,d
	ld	[hl+],a		;first_pulse_on
	ld	a,e
	ld	[hl+],a		;carrier_on
	ld	d,1
	ld	e,0		;num of marks & spaces

mark_space_count
	inc	e
	ld	d,0
mark_count
	inc	d		;d=num of carrier cycles
mc1	ld	a,[c]		;wait for current
	and	b		;pulse to go off
	jp	z,mc1


	REPT	13		;IR now off
	ld	a,[c]		;check to see if
	and	b		;this is a space
	jp	z,mark_count
	ENDR

	ld	a,d		;if we get here
	ld	[hl+],a		;we now see a space
				;mark_1
	ld	d,0

space_count

	REPT	8
	ld	a,[c]
	and	b
	jp	z,end_space_count
	ENDR

	inc	d		;if space>255 then end
	jp	nz,space_count

	ld	a,e
	ld	[m_s],a		;num of marks
	jp	end_recording_sequence

end_space_count

	ld	a,d
	ld	[hl+],a		;space1

	jp	mark_space_count

end_recording_sequence
	call	set_to_black


	call	decode_sony

	ret

decode_webtv
	call	blank_web_bits
	ld	hl,mark1+8	;all webtv signals are
	ld	de,web_bits	;the same to here
web_new_mark
	ld	a,[hl+]
	;cp	0		;finish if no more marks
	;ret	z
	ld	b,a
	srl	b		;divide by 32
	srl	b
	srl	b
	srl	b
	srl	b
	inc	b
web_mark_length
	ld	a,web_mark_symbol
	ld	[de],a
	inc	de
	dec	b
	jp	nz,web_mark_length
web_space
	ld	a,[hl+]
	cp	0		;finish if no more spaces
	ret	z
	ld	b,a
	srl	b		;divide by 32
	srl	b
	srl	b
	srl	b
	srl	b
	inc	b
web_space_length
	ld	a,web_space_symbol
	ld	[de],a
	inc	de
	dec	b
	jp	z,web_new_mark
	jp	web_space_length

web_byte_calculate0
	ld	b,8
	ld	hl,web_bits+1
wbc02	ld	a,[hl+]
	cp	web_mark_symbol
	scf
	jp	z,wbc01		;jump if space
	ccf
wbc01	rr	d		;d is web byte
	dec	b
	jp	nz,wbc02

	ld	a,d
	ld	[web_byte0],a
	ret

web_byte_mask
	xor	a
	ld	[web_byte1],a	;mask byte0
	ld	a,[web_byte0]
	res	7,a		;mask bit 7
	ld	[web_byte0],a
	ret

web_byte_calculate1
	ld	b,8
	ld	hl,web_bits+9
wbc12	ld	a,[hl+]
	cp	web_mark_symbol
	scf
	jp	z,wbc11		;jump if space
	ccf
wbc11	rr	d		;d is web byte
	dec	b
	jp	nz,wbc12

	ld	a,d
	ld	[web_byte1],a
	ret

blank_web_bits
	ld	b,20		;num of blanks
	ld	a,16		;blank char
	ld	hl,web_bits
web_bits_clear
	ld	[hl+],a
	dec	b
	jp	nz,web_bits_clear
	ret


decode_sony
	ld	a,[mark1]
	cp	$54			;valid header
	jp	c,decode_sony_error	;between $54-$5E
	cp	$5E
	jp	nc,decode_sony_error

	call	decode_webtv

	call	web_byte_calculate0
	call	web_byte_calculate1
	call	web_byte_mask


	ld	d,8		;decode first 8 bits
	ld	hl,mark1+2
ds1	ld	a,[hl+]
	cp	$25
	rr	b
	inc	hl
	dec	d
	jp	nz,ds1

	ld	a,b
	cpl
	ld	[sony0],a
	
	ld	d,8		;decode last 8 bits
ds2	ld	a,[hl+]
	cp	$25
	rr	b
	inc	hl
	dec	d
	jp	nz,ds2

	ld	a,b
	cpl
	ld	[sony1],a
	ret

decode_sony_error
	ld	a,$99
	ld	[sony1],a
	ret

set_to_red
	call	waitVB
	ld	a,%10001000
	ldh	[$68],a		;BCPS

	ld	a,%00011111	;palette
	ldh	[$69],a
	ld	a,%00111000
	ld	[$69],a
	ret

set_to_black
	call	waitVB
	ld	a,%10001000
	ldh	[$68],a		;BCPS
	ld	a,%00000000	;palette
	ldh	[$69],a
	ld	a,%00000000
	ldh	[$69],a
	ret

set_to_yellow
	call	waitVB
	ld	a,%10001000
	ldh	[$68],a		;BCPS
	ld	a,%01101100	;palette
	ldh	[$69],a
	ld	a,%00000000
	ldh	[$69],a
	ret

set_to_blue
	call	waitVB
	ld	a,%10001000
	ldh	[$68],a		;BCPS
	ld	a,%00000000	;palette
	ldh	[$69],a
	ld	a,%01111100
	ldh	[$69],a
	ret

clear_ram
	;ld	a,[joypad]	;clear ram if sel button
	;cp	$04
	;jp	z,cr1
	;ret
cr1	ld	b,0
	ld	a,0
	ld	hl,counter
cr2	ld	[hl+],a
	dec	b
	jp	nz,cr2
cr3	ld	[hl+],a
	dec	b
	jp	nz,cr3

	ret

initialize

.wait	ldh	a,[$44]		;LY LCDC compare
	cp	144
	jr	nc,.wait
	ld	a,0
	ldh	[$40],a		;LCDC lcd control

	ld	a,%10000000
	ldh	[$68],a		;BCPS
	ld	a,%00000000	;palette 0 0  bg
	ldh	[$69],a		;BCPD
	ld	a,%00000000
	ldh	[$69],a

	ld	a,%11111110	;palette 0 1
	ldh	[$69],a
	ld	a,%00011110
	ldh	[$69],a

	ld	a,%11111111	;palette 0 2  fg
	ldh	[$69],a
	ld	a,%01111111
	ldh	[$69],a

	ld	a,%00011111	;palette 0 3
	ldh	[$69],a
	ld	a,%00000000
	ldh	[$69],a

	ld	a,%00000000	;palette 1 0  bg
	ldh	[$69],a
	ld	a,%00000000
	ldh	[$69],a

	ld	a,%01100000	;palette 1 1
	ldh	[$69],a
	ld	a,%00011110
	ldh	[$69],a

	ld	a,%11100000	;palette 1 2  fg
	ldh	[$69],a
	ld	a,%00111111
	ldh	[$69],a

	ld	a,%11100000	;palette 1 3
	ldh	[$69],a
	ld	a,%01111111
	ldh	[$69],a

	ld	hl,Font
	ld	de,$8000
	ld	bc,20*8*2
	call	mem_Copy

	ld	b,$00
	ld	a,16
	ld	hl,$9800
cloop	ld	[hl+],a
	dec	b
	jp	nz,cloop
dloop	ld	[hl+],a
	dec	b
	jp	nz,dloop
hloop	ld	[hl+],a
	dec	b
	jp	nz,hloop

	ld	a,1
	ldh	[$4F],a		;VBK
	ld	b,0		;assign what palettes
	ld	a,%00000001	;to use on map
	ld	hl,$9800
eloop	ld	[hl+],a		;
	dec	b
	jp	nz,eloop
floop	ld	[hl+],a
	dec	b
	jp	nz,floop
gloop	ld	[hl+],a
	dec	b
	jp	nz,gloop
	ld	hl,$9982	;sony line
	ld	a,0
	ld	[hl+],a
	ld	[hl+],a
	ld	[hl+],a
	ld	[hl+],a
	ld	[hl+],a
	ld	a,0
	ldh	[$4F],a		;VBK


	ld	a,0
	ldh	[$42],a		;SCY Scroll Y
	ldh	[$43],a		;SCX Scroll X

	ld	a,%00000011
	ldh	[$47],a		;BGP

	ld	a,$00000000
	ldh	[$FF],a		;IE

	ld	a,%10010001
	ldh	[$40],a

	xor	a
	ld	[$FF24],a

	;ei
	ret

mem_Copy::
	inc	b
	inc	c
	jr	.skip
.loop	ld	a,[hl+]
	ld	[de],a
	inc	de
.skip	dec	c
	jr	nz,.loop
	dec	b
	jr	nz,.loop
	ret

Font:
	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`00010000
	DW	`00110000
	DW	`00010000
	DW	`00010000
	DW	`00010000
	DW	`00010000
	DW	`00111000
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`00000010
	DW	`01111100
	DW	`10000000
	DW	`10000000
	DW	`11111110
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`00000010
	DW	`00011100
	DW	`00000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`00001100
	DW	`00010100
	DW	`00100100
	DW	`01000100
	DW	`11111110
	DW	`00000100
	DW	`00000100
	DW	`00000000

	DW	`11111110
	DW	`10000000
	DW	`10000000
	DW	`11111100
	DW	`00000010
	DW	`00000010
	DW	`11111100
	DW	`00000000

	DW	`01111100
	DW	`10000000
	DW	`10000000
	DW	`11111100
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`11111110
	DW	`00000010
	DW	`00000100
	DW	`00001000
	DW	`00010000
	DW	`00010000
	DW	`00010000
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`01111110
	DW	`00000010
	DW	`00000010
	DW	`01111100
	DW	`00000000

	DW	`00111000
	DW	`01000100
	DW	`10000010
	DW	`11111110
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`00000000

	DW	`11111100
	DW	`10000010
	DW	`10000010
	DW	`11111100
	DW	`10000010
	DW	`10000010
	DW	`11111100
	DW	`00000000

	DW	`00111100
	DW	`01000010
	DW	`10000000
	DW	`10000000
	DW	`10000000
	DW	`01000010
	DW	`00111100
	DW	`00000000

	DW	`11111000
	DW	`10000100
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`10000100
	DW	`11111000
	DW	`00000000

	DW	`11111110
	DW	`10000000
	DW	`10000000
	DW	`11111000
	DW	`10000000
	DW	`10000000
	DW	`11111110
	DW	`00000000

	DW	`11111110
	DW	`10000000
	DW	`10000000
	DW	`11111000
	DW	`10000000
	DW	`10000000
	DW	`10000000
	DW	`00000000

	DW	`00000000	;16
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000

	DW	`11111111	;17 web mark
	DW	`10000001
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000

	DW	`00000000	;18 web space
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`10000001
	DW	`11111111
	DW	`00000000

	DW	`11111100	;19
	DW	`10000010	;P for parity
	DW	`10000010
	DW	`11111100
	DW	`10000000
	DW	`10000000
	DW	`10000000
	DW	`00000000


waitVB
	ldh	a,[$44]
	cp	144
	jp	nz,waitVB
	ret

write_to_screen

	ld	de,counter
	ld	hl,$9800
	call	display_byte

	ld	de,joypad
	ld	hl,$9840
	call	display_byte

	ld	de,first_pulse
	ld	hl,$9880
	call	display_byte

	ld	de,carrier_on
	ld	hl,$98C0
	call	display_byte

	ld	de,m_s
	ld	hl,$9900
	call	display_byte

	ld	de,sony1
	ld	hl,$9982
	call	display_byte

	ld	de,sony0
	ld	hl,$9985
	call	display_byte

	;ld	de,web_byte1
	;ld	hl,$9800+$20*15
	;call	display_byte

	ld	de,web_byte0
	ld	hl,$9800+$20*15+3
	call	display_byte


	ld	de,mark1
	ld	hl,$9809	;9809
	call	display_column

	ld	de,mark1+32
	ld	hl,$980F	;980F
	call	display_column

	ld	hl,web_line
	ld	a,[web_bits]
	ld	[hl+],a
	ld	a,[web_bits+1]
	ld	[hl+],a
	ld	a,[web_bits+2]
	ld	[hl+],a
	ld	a,[web_bits+3]
	ld	[hl+],a
	ld	a,[web_bits+4]
	ld	[hl+],a
	ld	a,[web_bits+5]
	ld	[hl+],a
	ld	a,[web_bits+6]
	ld	[hl+],a
	ld	a,[web_bits+7]
	ld	[hl+],a
	ld	a,[web_bits+8]
	ld	[hl+],a
	ld	a,[web_bits+9]
	ld	[hl+],a
	ld	a,[web_bits+10]
	ld	[hl+],a
	ld	a,[web_bits+11]
	ld	[hl+],a

	ret

intial_display
	call	waitVB
	xor	a
	ld	hl,$9800+$20*17+1	;line 18+1
id1	ld	[hl+],a
	inc	a
	cp	6
	jp	nz,id1
	ld	[hl+],a
	inc	hl
	ld	a,19
	ld	[hl],a
	ret

display_byte
	ld	a,[de]
	ld	b,a
	and	$F0
	swap	a
	ld	[hl+],a
	ld	a,b
	and	$0F
	ld	[hl],a
	ret

display_column
	ld	c,16
display_column_loop
	ld	a,[de]
	inc	de
	ld	b,a
	and	$F0
	swap	a
	ld	[hl+],a		;hi byte mark
	ld	a,b
	and	$0F
	ld	[hl+],a		;low byte
	inc	hl
	ld	a,[de]
	inc	de
	ld	b,a
	and	$F0
	swap	a
	ld	[hl+],a		;hi byte space
	ld	a,b
	and	$0F
	ld	[hl+],a		;low byte
	ld	a,$1B
	add	l
	jp	nc,dont_inc_h
	inc	h
dont_inc_h
	ld	l,a
	dec	c
	jp	nz,display_column_loop
	ret

keypad				;value in b
	ld	a,$20
	ldh	[$00],a		;p14 low
	ldh	a,[$00]
	ldh	a,[$00]
	cpl
	and	$0F
	swap	a
	ld	b,a
	ld	a,$10
	ldh	[$00],a
	ldh	a,[$00]
	ldh	a,[$00]
	ldh	a,[$00]
	ldh	a,[$00]
	ldh	a,[$00]
	ldh	a,[$00]
	cpl
	and	$0F
	or	b
	ld	b,a
	ld	a,[joypad]
	xor	b
	and	a
	ld	[joypadpulse],a
	ld	a,b
	ld	[joypad],a
	ld	a,$30
	ldh	[$00],a
	ret

;	SECTION "Cart_ram",DATA[$A000]
;d_up		DS	100
;d_down		DS	100
;d_right	DS	100
;d_left		DS	100


	SECTION	"GB_ram",BSS
counter:	DS	1
joypad:		DS	1
joypadpulse:	DS	1
test:		DS	1
web_byte0	DS	1
web_byte1	DS	1
web_bits	DS	20
sony1		DS	1
sony0		DS	1
m_s		DS	1
first_pulse	DS	1
carrier_on	DS	1
mark1		DS	200
